/*
===============================================================================
  NumTextField.java - an abstract Java class to implement text fields 
                      restricted to numerical input  

  Simon D. Levy              Brandeis University                           1998
===============================================================================
*/

import java.awt.*;
import java.awt.event.*;

public abstract class NumTextField extends TextField {

  // flag for input from
  // correct bad text
  static boolean fromKeyboard;

  // each sub-class must implement this method, which should return true if
  // number remains valid after adding character c to current string s,
  // false otherwise
  public abstract boolean checkCharAt(char c, int pos, String s);

  // add a TextListener to constrain input
  public void addListener() {

    // initial write to field is not from keyboard
    fromKeyboard = false;

    // add listener to reject non-digit characters
    addTextListener(new TextListener() {
      public void textValueChanged(TextEvent e) {
        if (fromKeyboard) {	  
          TextField tf = (TextField)e.getSource();
	  int p = getCaretPosition();
	  String s = tf.getText(), t = new String();
	  boolean okay = true;
	  for (int i=0; i<s.length(); ++i) {
	    char c = s.charAt(i), ca[] = {c};
	    if (checkCharAt(c, i, s))
	      t = t.concat(new String(ca));
	    else
	      okay = false;
	  }
	  if (!okay) {
	    fromKeyboard = false;
	    tf.setText(t);
	    tf.setCaretPosition(p>0?p-1:p);
	  }
	}

	else
	  fromKeyboard = true;

      } // textValueChanged

    }); // addTextListener

  } // addListener


  // constructors
  public NumTextField() {
    super();
    addListener();
  }
  public NumTextField(int columns) {
    super(columns);
    addListener();
  }
  public NumTextField(String text) {
    super(text);
    addListener();
  }
  public NumTextField(String text, int columns) {
    super(text, columns);
    addListener();
  }

} // class
